-- This file is a part of Falling Block Game 2.
-- 
-- Copyright (C) 2010 Jared Krinke.
-- 
-- Falling Block Game 2 is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

require("UI/UI.lua")
require("Fbg2/Common.lua")
require("Fbg2/GameLayer.lua")
require("Menus/CreditsMenu.lua")
require("Menus/HighScore.lua")
require("Menus/HighScoreEntryMenu.lua")
require("Menus/HighScoreMenu.lua")
require("Menus/OptionsMenu.lua")
require("Utility/MiscSettings.lua")

MainMenu = { }

function MainMenu.new()
    local mainForm = Form.newGrid(1, -200, 240, 400)
    local initialMode = MiscSettings.get("string", "mode")
    local initialLevel = MiscSettings.get("number", "level")
    local initialHeight = MiscSettings.get("number", "height")

    if isString(initialMode) then
        mainForm.mode = Fbg2.modes[initialMode]
    else
        mainForm.mode = 1
    end

    if isNumber(initialLevel) then
        mainForm.level = initialLevel
    else
        mainForm.level = 0
    end

    if isNumber(initialHeight) then
        mainForm.height = initialHeight
    else
        mainForm.height = 0
    end

    mainForm.creditsMenu = CreditsMenu.new()
    mainForm.highScoreEntryMenu = HighScoreEntryMenu.new()
    mainForm.highScoreMenu = HighScoreMenu.new()
    mainForm.optionsMenu = OptionsMenu.new()

    function mainForm.startNewGame(self)
        self.gameLayer:start(self.mode, self.level, self.height)
    end

    function mainForm.showHighScoreEntry(self, mode, height, score, cleared, level, entered)
        self.highScoreEntryMenu:show(mode, height, score, cleared, level, entered)
    end

    function mainForm.showHighScores(self, page)
        self.highScoreMenu:show(page)
    end

    function mainForm.showVideoOptions(self)
        self.videoMenu:show()
    end
    
    function mainForm.exit(self)
        exit()
    end

    -- Add game layer
    mainForm.gameLayer = GameLayer.new(function(valid, mode, height, score, cleared, level)
        local page = HighScore.getPage(mode, height)

        if valid and score > HighScore.getLowestScore(mode, height) then
            mainForm:showHighScoreEntry(mode, height, score, cleared, level, function() mainForm:showHighScores(page) end)
        else
            mainForm:showHighScores(page)
        end
    end)

    -- Form
    local levels = { "0", "1", "2", "3", "4", "5", "6", "7", "8", "9" }
    local heights = { "0", "1", "2", "3", "4", "5" }
    local heightChoice = Choice.new("Height", heights, function(self, choice) mainForm.height = 0 + choice end, true)
    local modeChoice = Choice.new("Mode", Fbg2.modeNames, function(self, choice)
        mainForm.mode = Fbg2.modes[choice]
        heightChoice:setActive(mainForm.mode == Fbg2.modes.Cleanup)
    end, true)

    local levelChoice = Choice.new("Level", levels, function(self, choice) mainForm.level = 0 + choice end, true)

    -- Select initial level
    modeChoice:select(Fbg2.modeNames[mainForm.mode])
    levelChoice:select("" .. mainForm.level)
    heightChoice:select("" .. mainForm.height)

    mainForm:add(ImageLabel.new("Images/Title.png", 640, 80, "center"))
    mainForm:add(Form.newNestedLeftFlow(1,
        Separator.new(),
        Button.new("Start New Game", function()
            -- Save selectsion
            MiscSettings.set("string", "mode", Fbg2.modeNames[mainForm.mode])
            MiscSettings.set("number", "level", mainForm.level)
            MiscSettings.set("number", "height", mainForm.height)
            MiscSettings.save()

            -- Start game
            mainForm:startNewGame()
        end, true),

        modeChoice,
        levelChoice,
        heightChoice,
        Separator.new(),
        Button.new("High Scores", function() mainForm:showHighScores() end),
        Button.new("Options", function() mainForm.optionsMenu:show()  end),
        Button.new("Credits", function() mainForm.creditsMenu:show() end, true),
        Separator.new(),
        Button.new("Quit", function() mainForm:exit() end)))

    -- Create layer
    local mainMenu = FormLayer.new(mainForm)
    
    -- Error handling
    function mainMenu.errorOccurred(self, message)
        print(message)
        console:show()
    end

    -- Console for debugging (TODO: remove eventually or only add to debug versions)
    -- TODO: Re-enable console
    function mainMenu.inputReceived(self, event, pressed)
        local handled = false

        if event == "openConsole" and pressed then
            console:show()
            handled = true
        else
            handled = mainForm:inputReceived(event, pressed)
        end

        return handled
    end

    return mainMenu
end

