-- This file is a part of Falling Block Game 2.
-- 
-- Copyright (C) 2010 Jared Krinke.
-- 
-- Falling Block Game 2 is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

require("UI/UI.lua")
require("Utility/ForEach.lua")
require("Utility/SortedList.lua")
require("Fbg2/Common.lua")
require("Settings/VideoSettings.lua")
require("Utility/MiscSettings.lua")

Video.setMode(videoSettings.width, videoSettings.height, videoSettings.fullscreen)

VideoMenu = { }

function VideoMenu.new()
    local videoForm = Form.newGrid(1, Fbg2.formX, Fbg2.formY, Fbg2.formWidth)

    local function resolutionToName(width, height)
        return width .. "x" .. height
    end

    local modes = { "Windowed", "Fullscreen" }
    local animateModes = { "Off", "On" }
    local animateModeToValue = { [animateModes[1]] = false, [animateModes[2]] = true }
    local valueToAnimateMode = { [false] = animateModes[1], [true] = animateModes[2] }

    local function modeToName(fullscreen)
        if fullscreen then
            return modes[2]
        else
            return modes[1]
        end
    end

    local modeToFullscreen = {
        [modes[1]] = false,
        [modes[2]] = true,
    }

    local function resolutionCompare(a, b)
        if a.height ~= b.height then
            return a.height - b.height
        else
            return a.width - b.width
        end
    end

    local resolutionList = SortedList.new(resolutionCompare, Video.getModes())
    local resolutionNames = { }
    local resolutionNameToResolution = { }

    resolutionList:forEach(function(resolution)
        local resolutionName = resolutionToName(resolution.width, resolution.height)

        resolutionNames[#resolutionNames + 1] = resolutionName
        resolutionNameToResolution[resolutionName] = resolution
    end, Video.getModes())

    function videoForm.refresh(self)
        self.resolutionChoice:select(resolutionToName(Video.getPixelWidth(), Video.getPixelHeight()), true)
        self.modeChoice:select(modeToName(Video.getFullscreen()), true)
        self.animateChoice:select(valueToAnimateMode[Fbg2.getAnimate()])
        -- TODO: Select the first item
    end

    videoForm.resolutionChoice  = Choice.new("Resolution", resolutionNames, function(self, choice) end)
    videoForm.modeChoice        = Choice.new("Mode", modes, function(self, choice) end)
    videoForm.animateChoice     = Choice.new("Animations", animateModes, function(self, choice) end)

    -- UI
    videoForm:add(Title.new("Video Options"))

    videoForm:add(Form.newNestedLeftFlow(1,
        Separator.new(),
        videoForm.resolutionChoice,
        videoForm.modeChoice,
        videoForm.animateChoice,
        Separator.new(),
        Button.new("Apply Changes", function()
            -- Set mode
            local resolution = resolutionNameToResolution[videoForm.resolutionChoice:getChoice()]
            local fullscreen = modeToFullscreen[videoForm.modeChoice:getChoice()]
    
            Video.setMode(resolution.width, resolution.height, fullscreen)
    
            -- Verify mode and save settings
            if Video.getPixelWidth() == resolution.width and Video.getPixelHeight() == resolution.height and Video.getFullscreen() == fullscreen then
                -- Save video settings
                File.createDirectory("Settings")
    
                local file = File.openWrite("Settings/VideoSettings.lua")
    
                file:write("videoSettings = {")
                file:write("    width = " .. Video.getPixelWidth() .. ",")
                file:write("    height = " .. Video.getPixelHeight() .. ",")
                if Video.getFullscreen() then
                    file:write("    fullscreen = true,")
                else
                    file:write("    fullscreen = false,")
                end
                file:write("}")
    
                file:close()

                -- Save other related settings
                MiscSettings.set("boolean", "animate", animateModeToValue[videoForm.animateChoice:getChoice()])
                MiscSettings.save()
            end
        end),
        Separator.new(),
        Button.new("Back", function() Layers.pop() end)))

    local videoMenu = FormLayer.new(videoForm)

    function videoMenu.formShown(self)
        videoForm:refresh()
    end

    return videoMenu
end

